@@ -25,15 +25,14 @@
 
 
 #include "gegl-buffer.h"
+#include "opencl/gegl-cl.h"
 
 G_BEGIN_DECLS
 
-#define GEGL_TYPE_OPERATION            (gegl_operation_get_type ())
-#define GEGL_OPERATION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_OPERATION, GeglOperation))
 #define GEGL_OPERATION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  GEGL_TYPE_OPERATION, GeglOperationClass))
-#define GEGL_IS_OPERATION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_OPERATION))
 #define GEGL_IS_OPERATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  GEGL_TYPE_OPERATION))
 #define GEGL_OPERATION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  GEGL_TYPE_OPERATION, GeglOperationClass))
+/* The rest is in gegl-types.h */
 
 typedef struct _GeglOperationClass GeglOperationClass;
 
@@ -62,24 +61,33 @@ struct _GeglOperation
  * See <a href='gegl-plugin.h.html'>gegl-plugin.h</a> for details.
  */
 
-#define MAX_PROCESSOR 4
-
-void gegl_operation_class_add_processor (GeglOperationClass *cclass,
-                                         GCallback           process,
-                                         const gchar        *string);
+/* the level at which is being operated is stored in the context,
+*/
 
 struct _GeglOperationClass
 {
   GObjectClass    parent_class;
 
-  const gchar    *name;        /* name(string) used to create/indetify
-                                    this type of operation in GEGL*/
+  const gchar    *name;        /* name(string) used to create/identify
+                                  this type of operation in GEGL, must be
+                                  set through gegl_operation_class_set_key(s) */
   const gchar    *compat_name; /* allows specifying an alias that the op is
-                                  also known as */
-  const gchar    *description; /* textual description of the operation */
-  const gchar    *categories;  /* a colon seperated list of categories */
+                                  also known as, must be set through
+                                  gegl_operation_class_set_key(s) */
+  GHashTable     *keys;        /* hashtable used for storing meta-data about an op */
 
-  gboolean        no_cache;    /* do not create a cache for this operation */
+  guint           no_cache      :1;  /* do not create a cache for this operation */
+  guint           opencl_support:1;
+  guint           want_in_place:1; /* if possible to use for in-place
+                                      processing, making output buffer =
+                                      input buffer.
+                                      */
+  guint           threaded:1;  /* do threaded processing if possible,
+                                  some base classes have special logic
+                                  to accelerate rendering; this allows opting in/out
+                                  in the sub-classes of these.
+                                */
+  guint64         bit_pad:60;
 
   /* attach this operation with a GeglNode, override this if you are creating a
    * GeglGraph, it is already defined for Filters/Sources/Composers.
@@ -127,25 +135,27 @@ struct _GeglOperationClass
   gboolean      (*process)                   (GeglOperation        *operation,
                                               GeglOperationContext *context,
                                               const gchar          *output_pad,
-                                              const GeglRectangle  *roi);
+                                              const GeglRectangle  *roi,
+                                              gint                  level);
 
   /* The node providing data for a specific location within the operations
    * output. The node is responsible for delegating blame to one of it's
    * inputs taking into account opacity and similar issues.
    *
-   * XXX: What is GeglNode doing in this part of the API?
-   * Does this recurse?, perhaps we should only point out which pad the
-   * data is coming from?
+   * XXX: What is GeglNode doing in this part of the API?,
+   * perhaps we should only point out which pad the data is coming from?
    */
   GeglNode*     (*detect)                    (GeglOperation       *operation,
                                               gint                 x,
                                               gint                 y);
+
+  GeglClRunData *cl_data;
+
+  gpointer      pad[9];
 };
 
 
 
-GType           gegl_operation_get_type        (void) G_GNUC_CONST;
-
 GeglRectangle   gegl_operation_get_invalidated_by_change
                                              (GeglOperation *operation,
                                               const gchar   *input_pad,
@@ -181,8 +191,9 @@ void            gegl_operation_attach        (GeglOperation *operation,
 void            gegl_operation_prepare       (GeglOperation *operation);
 gboolean        gegl_operation_process       (GeglOperation *operation,
                                               GeglOperationContext *context,
-                                              const gchar   *output_pad,
-                                              const GeglRectangle *roi);
+                                              const gchar          *output_pad,
+                                              const GeglRectangle  *roi,
+                                              gint                  level);
 
 /* create a pad for a specified property for this operation, this method is
  * to be called from the attach method of operations, most operations do not
@@ -206,30 +217,86 @@ const Babl *    gegl_operation_get_format    (GeglOperation *operation,
 const gchar *   gegl_operation_get_name      (GeglOperation *operation);
 
 
+/* checks the incoming Babl format on a given pad, can be used in the prepare
+ * stage to make format dependent decisions
+ */
+const Babl  * gegl_operation_get_source_format (GeglOperation *operation,
+                                                const gchar   *padname);
+
 /* retrieves the node providing data to a named input pad */
-GeglNode      * gegl_operation_get_source_node (GeglOperation *operation,
+GeglNode    * gegl_operation_get_source_node   (GeglOperation *operation,
                                                 const gchar   *pad_name);
 
-GParamSpec ** gegl_list_properties (const gchar *operation_type,
-                                    guint       *n_properties_p);
+/* API to change  */
+void          gegl_operation_class_set_key     (GeglOperationClass *klass,
+                                                const gchar *key_name,
+                                                const gchar *key_value);
+
+const gchar * gegl_operation_class_get_key     (GeglOperationClass *operation_class,
+                                                const gchar        *key_name);
+
+void          gegl_operation_class_set_keys    (GeglOperationClass *klass,
+                                                const gchar        *key_name,
+                                                ...);
 
 
-/* invalidate a specific rectangle, indicating the any computation depending
+void          gegl_operation_set_key           (const gchar *operation_type,
+                                                const gchar *key_name,
+                                                const gchar *key_value);
+
+
+gboolean      gegl_operation_use_opencl         (const GeglOperation *operation);
+
+gboolean
+gegl_operation_use_threading (GeglOperation *operation,
+                              const GeglRectangle *roi);
+
+/* Invalidate a specific rectangle, indicating the any computation depending
  * on this roi is now invalid.
  *
  * @roi : the region to blank or NULL for the nodes current have_rect
  * @clear_cache: whether any present caches should be zeroed out
  */
-void     gegl_operation_invalidate            (GeglOperation       *operation,
-                                               const GeglRectangle *roi,
-                                               gboolean             clear_cache);
+void     gegl_operation_invalidate       (GeglOperation       *operation,
+                                          const GeglRectangle *roi,
+                                          gboolean             clear_cache);
+
+gboolean gegl_operation_cl_set_kernel_args (GeglOperation *operation,
+                                            cl_kernel      kernel,
+                                            gint          *p,
+                                            cl_int        *err);
 
 /* internal utility functions used by gegl, these should not be used
  * externally */
-gboolean gegl_operation_calc_need_rects      (GeglOperation       *operation,
-                                              gpointer             context_id);
-void     gegl_operation_path_prop_changed    (GeglPath            *path,
-                                              GeglOperation       *operation);
+gboolean gegl_can_do_inplace_processing      (GeglOperation       *operation,
+                                              GeglBuffer          *input,
+                                              const GeglRectangle *result);
+
+/**
+ * gegl_object_set_has_forked: (skip)
+ * @object: Object to mark
+ *
+ * Mark an object as "forked", indicating that it should not be modified
+ * in place by gegl operations. This should only be used inside of the
+ * process method of a GeglOperation subclass.
+ */
+void     gegl_object_set_has_forked       (GObject *object);
+
+/**
+ * gegl_object_get_has_forked: (skip)
+ * @object: Object to check
+ *
+ * Returns TRUE if the object has been marked as "forked".
+ */
+gboolean  gegl_object_get_has_forked      (GObject *object);
+
+/**
+ * gegl_temp_buffer:
+ *
+ * Returns a singleton scratch buffer for use with multi-threaded processing
+ * dispatch.
+ */
+guchar    *gegl_temp_buffer (int no, int min_size);
 
 G_END_DECLS
 
